#!/usr/bin/perl
#
# abstractart2.cgi:	produce a single mondrian style image of random lines
#			and filled boxes, avoiding overlaps and showing all
#			the stages
#
#	(C) Duncan C. White, 23rd February 1999
#
use strict;
use GD;
use webprompt;
use col;
use vars qw(@avr @newavr @drawlist $gap);


$gap = 10;


sub min ($$) { return $_[ $_[0] > $_[1] ]; }
sub max ($$) { return $_[ $_[0] < $_[1] ]; }
sub intrand ($) { return $_[0]<1?0:int(rand($_[0])); }


&webprompt::get(
	"abstractart2", "Mondrian-style Abstract Art",
	\&giveback, \&validate, \&createimg,
	qw(X Y Palette=RBY|RBYB|RBYBB|green MinBoxes MaxBoxes MinThickness MaxThickness) );

exit 0;


sub giveback ($$$$$$$)
{
	my( $x, $y, $palette, $minboxes, $maxboxes, $minthick, $maxthick ) = @_;
	
	$p'x = $x;
	$p'y = $y;
	$p'palette = $palette;
	$p'minboxes = $minboxes;
	$p'maxboxes = $maxboxes;
	$p'minthick = $minthick;
	$p'maxthick = $maxthick;

	return @_;
}


sub validate ()
{
	return "" if $p'x > 50 && $p'y > 50 && $p'minboxes > 0 &&
		$p'maxboxes >= $p'minboxes && $p'minthick > 0 &&
		$p'maxthick >= $p'minthick;
	return "These values are not sensible";
}


sub createimg ()
{
	$p'XD = $p'x/10;
	$p'YD = $p'y/10;

	$p'im = new GD::Image($p'x--, $p'y--);

	col::read( $p'im, $p'palette );

	#$p'im->transparent($col::white);
	$p'im->interlaced('true');

	my $boxes = $p'minboxes + intrand(1+$p'maxboxes-$p'minboxes);

	# list of available rectangles
	@avr = ( [ $gap, $gap, $p'x-$gap, $p'y-$gap ] );

	#print "Let's try $boxes boxes, if they'll fit\n";

	@drawlist = ();				# list of boxes to draw

	for( my $i = 0; $i < $boxes && @avr > 0; $i++ )
	{
		#print "Trying box $i\n";
		onebox();
	}

	redraw();

	return $p'im;
}


sub onebox ()
{
	my $box = splice( @avr, intrand(@avr), 1 );

	my ( $xmin, $ymin, $xmax, $ymax ) = @$box;

	# print "chosen rect with xmin=$xmin, ymin=$ymin\n";

	my $rw = 1 + $xmax - $xmin - $p'XD;
	my $rh = 1 + $ymax - $ymin - $p'YD;

	my $x1 = $xmin + intrand($rw-5);
	my $y1 = $ymin + intrand($rh-5);

	my $x2 = min( $x1 + $p'XD + intrand($p'XD), $xmax );
	my $y2 = min( $y1 + $p'XD + intrand($p'YD), $ymax );

	my $col = $col'colour[intrand(@col'colour)];

	my $thick = $p'minthick + intrand(1+$p'maxthick-$p'minthick);

	push( @drawlist, [ $x1, $y1, $x2, $y2, $thick, $col ] );

	# now clip all the existing available rectangles against the
	# new rectangle lines (x=x1, y=y1, x=x2, y=y2) keeping only
	# the parts "outside" the new rectangle regions

	@newavr = ();

	foreach my $r (@avr)
	{
		my ( $rx1, $ry1, $rx2, $ry2 ) = @$r;

		my $x1inrange = $rx1 <= $x1 && $x1 <= $rx2;
		my $x2inrange = $rx1 <= $x2 && $x2 <= $rx2;
		my $y1inrange = $ry1 <= $y1 && $y1 <= $ry2;
		my $y2inrange = $ry1 <= $y2 && $y2 <= $ry2;

		if( $x1inrange || $x2inrange )
		{
			if( $x1inrange && $x2inrange )
			{
				addrect( $rx1, $ry1, $x1-$gap, $ry2 );

				$rx1 = $x2+$gap; # clip rx1 to right of x2

			} elsif( $x1inrange )
			{
				$rx2 = $x1-$gap; # clip rx2 to left of x1

			} else # if( $x2inrange )
			{
				$rx1 = $x2+$gap; # clip rx1 to right of x2
			}
		}
		if( $y1inrange || $y2inrange )
		{
			if( $y1inrange && $y2inrange )
			{
				addrect( $rx1, $ry1, $rx2, $y1-$gap );
				$ry1 = $y2+$gap; # clip ry1 to below y2

			} elsif( $y1inrange )
			{
				$ry2 = $y1-$gap; # clip ry2 to above y1
			} else # if( $y2inrange )
			{
				$ry1 = $y2+$gap; # clip ry1 to below y2
			}
		}
		addrect( $rx1, $ry1, $rx2, $ry2 );
	}

	addrect( $xmin, $ymin, $x1-$gap, $y1-$gap );
	addrect( $x2+$gap, $ymin, $xmax, $y1-$gap );
	addrect( $xmin, $y2+$gap, $x1-$gap, $ymax );
	addrect( $x2+$gap, $y2+$gap, $xmax, $ymax );

	@avr = @newavr;
}


sub redraw ()
{
	$p'im->filledRectangle( 0, 0, $p'x, $p'y, $col'white );

	foreach my $r (@drawlist)
	{
		my ( $x1, $y1, $x2, $y2, $thick, $col ) = @$r;

		my $th = $thick-1;

		$p'im->filledRectangle( 0, $y1, $p'x, $y1+$th,
			$col'black );
		$p'im->filledRectangle( 0, $y2-$th, $p'x, $y2,
			$col'black );
		$p'im->filledRectangle( $x1, 0, $x1+$th, $p'y,
			$col'black );
		$p'im->filledRectangle( $x2-$th, 0, $x2, $p'y,
			$col'black );

		$p'im->filledRectangle( $x1 + $thick, $y1 + $thick,
			$x2 - $thick, $y2 - $thick, $col );
	}

	#showavailable();
}


sub showavailable ()
{
	my $i = 0;

	foreach my $r (@avr)
	{
		my ( $rx1, $ry1, $rx2, $ry2 ) = @$r;

		# print "rx1=$rx1, ry1=$ry1, rx2=$rx2, ry2=$ry2\n";

		$p'im->filledRectangle( $rx1, $ry1, $rx2, $ry2, $col'cyan );

		$p'im->string( gdMediumBoldFont(), $rx1+5, $ry1+5, $i++,
			$col'black );
	}
}


sub addrect ($$$$)
{
	my( $x1, $y1, $x2, $y2 ) = @_;

	if( $x1 > $x2 )
	{
		my $t = $x1;
		$x1 = $x2;
		$x2 = $t;
	}
	if( $y1 > $y2 )
	{
		my $t = $y1;
		$y1 = $y2;
		$y2 = $t;
	}

	push( @newavr, [ $x1, $y1, $x2, $y2 ] ) if
		$x2 - $x1 > $gap && $y2 - $y1 > $gap;
}
