#
# webprompt.pm
#
#	framework for prompting a user for information and then
#	producing an image and displaying it.
#
#	the clever bit: does this either via the web or command line
#	arguments, depending on whether or not you give it any cmd
#	line arguments..
#
#	(C) Duncan C. White, 31st August 2000
#

package webprompt;


use CGI qw(:standard);
use GD;
use IO::Handle;
use strict;
use constant VIEWER => "xv";


sub get ($$$$@)
{
	my( $name, $title, $modify, $validate, $draw, @fields ) = @_;

	my $filename = "$$.png";

	my ( $reffield, $refmenu ) = splitequals( @fields );

	my $web = @ARGV == 0;

	if( $web )
	{
		my( @v, $validmesg );

		print	header(),
			start_html($title),
			h1($title),
			start_form();

		if( param() )
		{
			@v = getvalues( @$reffield );
			@v = &$modify( @v );
			setvalues( $reffield, @v );
			$validmesg = &$validate();
		}

		my $paramtable = paramtable( $reffield, $refmenu );

		my $image = "";

		if( param() )
		{
			if( $validmesg eq "" )
			{
				my $im = &$draw();
				save( $filename, $im );
				$image = img( {src=>$filename} );
			} else
			{
				$image = h2($validmesg);
			}
		}

		print table({-border=>undef},
			  Tr({-align=>"center"},
			  [
			     td( {-width=>"50%"}, [ $paramtable, $image ] )
			  ]
			  )
		       );
		print end_form(), end_html();
		cleanup( $filename );
	} else
	{
		die "Usage: $name ".join(' ',@fields)."\n" unless
			@ARGV == @fields;
		my @v = &$modify( @ARGV );
		my $mesg = &$validate();
		die "$name: $mesg\n" unless $mesg eq "";
		my $im = &$draw( );
		save( $filename, $im );
		system( VIEWER, $filename );
		unlink( $filename );
	}
}


sub save ($$)
{
	my( $filename, $im ) = @_;

	open( OUT, "> $filename" ) ||
		die "can't open $filename\n";

	binmode OUT;
	print OUT $im->png;
	close OUT;
}


sub splitequals (@)
{
	my @lhs = ();
	my %rhs = ();
	foreach my $f (@_)
	{
		my $rest;
		if( $f =~ /=/ )
		{
			( $f, $rest ) = split( /=/, $f, 2 );
			$rhs{$f} = $rest;
		}
		push( @lhs, $f );
	}

	return ( \@lhs, \%rhs );
}


sub getvalues (@)
{
	my @v = ();

	foreach my $f (@_)
	{
		push( @v, param($f) );
	}
	return @v;
}


sub setvalues ($@)
{
	my( $fref, @v ) = @_;

	foreach my $f (@$fref)
	{
		my $v = shift @v;
		if( param($f) ne $v )
		{
			param( $f, $v );
		}
	}
}


sub paramtable ($$)
{
	my( $reffield, $refmenu ) = @_;

	my $table = "";
	foreach my $f (@$reffield)
	{
		my $rest;
		if( $rest = $refmenu->{$f} )
		{
			$rest = makemenu( $f, split( /\|/, $rest ) );
		} else
		{
			$rest = textfield($f);
		}
		my $name = th(b("$f:"));
		my $field = td($rest);
		$table .= "  " . Tr({-align=>"left"}, "$name $field") . "\n";
	}
	$table .= "  " . Tr( td( [ submit(-value=>"Draw"), reset() ] ) ) . "\n";
	$table = "\n" . table( $table ) . "\n";

	return $table;
}


sub makemenu ($@)
{
	my( $name, @v ) = @_;

	my $option = "";
	foreach my $v (@v)
	{
		my $selected = param($name) eq $v ? "-selected" : "";
		$option .= "\n". option(
			{ -value => $v, $selected => 1 }, $v );
	}
	return Select( {-name=>$name}, $name, $option );
}


sub cleanup ($)
{
	my( $filename ) = @_;

	STDOUT->autoflush(1);

	if( fork == 0 )	# child
	{
		close( STDOUT );
		sleep( 20 );
		unlink( $filename );
	}
}

1;
